#
#    NMIS.pm - NMIS Perl Package - Network Mangement Information System
#    Copyright (C) 2000,2001 Sinclair InterNetworking Services Pty Ltd
#    Keith Sinclair <keith@sinclair.org.au>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
package NMIS;

use lib "/usr/local/rrdtool/lib/perl";

require 5;
use BER;
use SNMP_Session;
use SNMP_Simple;
use SNMP_MIB;
use strict;
use RRDs;
use Time::ParseDate;
use func;
use csv;

use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);

use Exporter;

$VERSION = "2.00";

@ISA = qw(Exporter);

@EXPORT = qw(	
		loadConfiguration
		loadInterfaceFile
		loadSystemFile
		loadLinkDetails
		loadNodeDetails
		loadRMENodes
		writeNodesFile
		loadEventState
		loadEventStateTime
		eventHashTime
		eventHash
		eventExist
		checkEvent
		logEvent
		eventAdd
		eventAck
		notify
		sendEmail
		sendPagerMessage
		writeEventState
		runEventDebug
		summaryStats
		getGroupSummary
		colorHighGood
		colorPort
		colorLowGood
		colorResponseTime
		thresholdResponse
		thresholdLowPercent
		thresholdHighPercent
		thresholdHighPercentLoose
		thresholdMemory
		thresholdInterfaceUtil
		thresholdInterfaceAvail
		loadInterfaceTypes
		eventLevel
		eventPolicy
		escalationPolicy
		overallNodeStatus
		logMessage
		createInterfaceInfo
		loadInterfaceInfo
		outageAdd
		outageDelete
		outageLoad
		outageCheck
		outageCheckHash
		convertTime
		convertTimeLength
	);

@EXPORT_OK = qw(	%config 
			%interfaceTable 
			%ifDescrTable 
			%systemTable 
			%eventTable
			%nodeTable
			%linkTable
			$eventCount
			$version
		);

# Set the default file locations if not specified in configuration file.
my %config;

# Nice Variables for Interface Stuff
my %interfaceTable;
my $interfaceTableNum;
my %ifDescrTable;
my %ifTypeDefs;

my $eventCount;

# System Info Hash
my %systemTable;

# Interface Information Table
my %interfaceInfo;

# Outage Table 
my %outageTable;

# A hash of all the node details
my %nodeTable;
my %nodeCount;
my %groupTable;

# Event table for groovy events
my %eventTable;

# A hash of all the link details
my %linkTable;

# Debug Option
my $debug = "false";

sub loadConfiguration {
	my $file = shift;

	my $name;
	my $value;
	my $key;

	$NMIS::config{file} = $file;

	open(DATAFILE, "$file") or warn returnTime." loadConfiguration, Cannot open $file. $!\n";
	while (<DATAFILE>) {
		chomp;
		# Line does not start with # must be configuration option
		if ( $_ !~ /^#|^ |^\t/ ) { 
			($name,$value) = split("=", $_);
			$name =~ s/;//g;
			$NMIS::config{$name} = $value;
		}
	}
	close(DATAFILE);

	# check for config variables and process each config element again.
	foreach $key (%NMIS::config) {
		if ( $key =~ /^<.*>$/ ) {
			if ($NMIS::debug>8) { print returnTime." Found a key to change $key\n"; }
			foreach $value (%NMIS::config) {
				if ( $NMIS::config{$value} =~ /<.*>/ ) {
					if ($NMIS::debug==9) { print "\tabout to change $value to $NMIS::config{$value}, $key, $NMIS::config{$key}\n"; }
					$NMIS::config{$value} =~ s/$key/$NMIS::config{$key}/;
				}
			}
		}
	}
}

sub loadInterfaceFile {
	my $node = shift;
	my $interfaceFile="$NMIS::config{'<nmis_var>'}/$node-interface.dat";
	
	undef %NMIS::interfaceTable;
	if ( -f $interfaceFile ) {
		%NMIS::interfaceTable = &loadCSV($interfaceFile,"interface","\t");
	}
	else {
  		logMessage("loadInterfaceFile, $node, Cannot open interface file $interfaceFile");
  		print returnTime." $node Cannot open interface file=$interfaceFile, this may not be a bad thing.\n";
	}
	# Now we have a nicely populated Interface Table we can do other things
}

sub loadSystemFile {
	my $node = shift;

	undef %NMIS::systemTable;

	my $systemFile="$NMIS::config{'<nmis_var>'}/$node.dat";
	my ($name,$value);

	if ( -f $systemFile ) { 
		open(DATAFILE, "$systemFile") or warn returnTime." loadSystemFile, Cannot open $systemFile. $!\n";
		while (<DATAFILE>) {
			chomp;
			# Does the line from configfile have a comment?
			if ( $_ !~ /^#/ ) {
				($name,$value) = split "=", $_;
				$NMIS::systemTable{$name} = $value;
			}
		}
		close(DATAFILE);
	}
	# Now we have a nicely populated Interface Table we can do other things
}
			   
sub loadLinkDetails {
	%NMIS::linkTable = &loadCSV($NMIS::config{Links_Table},$NMIS::config{Links_Key},"\t");
} #sub loadLinkDetails

sub loadNodeDetails {
	my $node;
	my $nodeType;
	# Load the CSV first
	%NMIS::nodeTable = &loadCSV($NMIS::config{Nodes_Table},$NMIS::config{Nodes_Key},"\t");
	# Bit of quick post processing on the table.
	foreach $node (keys %NMIS::nodeTable) {
		$NMIS::groupTable{$NMIS::nodeTable{$node}{group}} = $NMIS::nodeTable{$node}{group};
		if ( $NMIS::nodeCount{run} ne "true" ) {
			++$NMIS::nodeCount{total};
			$nodeType = $NMIS::nodeTable{$node}{net}.$NMIS::nodeTable{$node}{role};
			++$NMIS::nodeCount{$nodeType};
			++$NMIS::nodeCount{$NMIS::nodeTable{$node}{group}};
		}
	}
	$NMIS::nodeCount{run} = "true";
}

sub loadRMENodes {

	my $file = shift;

	my $ciscoHeader = "Cisco Systems NM";
	my @nodedetails;
	my @statsSplit;
	my $nodeType;

	open(DATAFILE, "$file") or warn returnTime." loadRMENodes, Cannot open $file. $!\n";
	while (<DATAFILE>) {
	        chomp;
		# Don't want comments 
	        if ( $_ !~ /^\;|^$ciscoHeader/ ) {
			# whack all the splits into an array
			(@nodedetails) = split ",", $_;
		
			# check that the device is to be included in STATS
			$nodedetails[4] =~ s/ //g;
			@statsSplit = split(":",$nodedetails[4]);
			if ( $statsSplit[0] =~ /t/ ) {
				$nodedetails[0] =~ s/\.macbank//g;
				$NMIS::nodeTable{$nodedetails[0]}{node} = $nodedetails[0];
				$NMIS::nodeTable{$nodedetails[0]}{community} = $nodedetails[1];
				$NMIS::nodeTable{$nodedetails[0]}{net} = $statsSplit[1];
				$NMIS::nodeTable{$nodedetails[0]}{devicetype} = $statsSplit[2];
				# Convert role c, d or a to core, distribution or access
				if ( $statsSplit[3] eq "c" ) { $NMIS::nodeTable{$nodedetails[0]}{role} = "core"; }
				elsif ( $statsSplit[3] eq "d" ) { $NMIS::nodeTable{$nodedetails[0]}{role} = "distribution"; }
				elsif ( $statsSplit[3] eq "a" ) { $NMIS::nodeTable{$nodedetails[0]}{role} = "access"; }
				# Convert collect t or f to  true or false
				if ( $statsSplit[4] eq "t" ) { $NMIS::nodeTable{$nodedetails[0]}{collect} = "true"; }
				elsif ( $statsSplit[4] eq "f" ) { $NMIS::nodeTable{$nodedetails[0]}{collect} = "false"; }
			}
		}
	}
	close(DATAFILE);
}

sub writeNodesFile {
	my $file = shift;
	if ($NMIS::debug) { print returnTime." Writing the Nodes File\n"; }
	&writeCSV(%NMIS::nodeTable,$file,"\t");
}

sub writeEventState {
	my $event_hash;

	loadNodeDetails;
	if ($NMIS::debug==9) {
		print "Current Event State Table:\n";
		print "StartTime,LastChange,Node,Event,Event_Level,Ack,Details\n";
	}
	open(OUTFILE, ">$NMIS::config{event_file}") or warn returnTime." writeEventState, Couldn't open file $NMIS::config{event_file}. $!\n";
	foreach $event_hash (sort ( keys (%NMIS::eventTable) ) )  {
		#print STDERR "$NMIS::eventTable{$event_hash}{node} $NMIS::eventTable{$event_hash}{event} $NMIS::eventTable{$event_hash}{details} $NMIS::eventTable{$event_hash}{ack}\n";
		if ( ($NMIS::debug==9) and $NMIS::eventTable{$event_hash}{current} eq "true" ) {
			print "$event_hash,";
			print "current=$NMIS::eventTable{$event_hash}{current},";
			print &returnDateStamp($NMIS::eventTable{$event_hash}{startdate}).",";
			print &returnDateStamp($NMIS::eventTable{$event_hash}{lastchange}).",";
			print "$NMIS::eventTable{$event_hash}{node},";
			print "$NMIS::eventTable{$event_hash}{event},";
			print "$NMIS::eventTable{$event_hash}{event_level},";
			print "$NMIS::eventTable{$event_hash}{details},";
			print "$NMIS::eventTable{$event_hash}{ack},";
			print "$NMIS::eventTable{$event_hash}{escalate}\n";

	        }
		#Only write non blank events and that the node exists or not
		# and only loads events for device that exist.
		if ( 	$NMIS::eventTable{$event_hash}{current} eq "true" 
				# this check only allows for devices in the node table to have entries in the event table
				# but now I want to have the ability to add events to in the NMIS event table to other devices.
				#and $NMIS::eventTable{$event_hash}{node} eq $NMIS::nodeTable{$NMIS::eventTable{$event_hash}{node}}{node}
				and $NMIS::eventTable{$event_hash}{startdate} ne "" 
				and $NMIS::eventTable{$event_hash}{lastchange} ne "" 
				and $NMIS::eventTable{$event_hash}{node} ne "" 
				and $NMIS::eventTable{$event_hash}{event} ne "" 
				and $NMIS::eventTable{$event_hash}{event_level} ne "" 
		) {
			print OUTFILE "$NMIS::eventTable{$event_hash}{startdate},";
			print OUTFILE "$NMIS::eventTable{$event_hash}{lastchange},";
			print OUTFILE "$NMIS::eventTable{$event_hash}{node},";
			print OUTFILE "$NMIS::eventTable{$event_hash}{event},";
			print OUTFILE "$NMIS::eventTable{$event_hash}{event_level},";
			print OUTFILE "$NMIS::eventTable{$event_hash}{details},";
			print OUTFILE "$NMIS::eventTable{$event_hash}{ack},";
			print OUTFILE "$NMIS::eventTable{$event_hash}{escalate}\n";
 		}
	} # foreach $linkname
	close(OUTFILE); 
}

sub loadEventState {
	my @eventdetails;
	my $node;
	my $event;
	my $level;
	my $details;
	my $event_hash;

	undef %NMIS::eventTable;

	$NMIS::eventCount = 0;

	open(DATAFILE, "<$NMIS::config{event_file}") or warn returnTime." loadEventState, Couldn't open file $NMIS::config{event_file}. $!\n";
	# File format is startdate,lastchange,node,event,event_level,ack,details
	while (<DATAFILE>) {
	        chomp;
		++$NMIS::eventCount;
		# whack all the splits into an array
		@eventdetails = split ",", $_;

		# using temp variables for readability
		$node = $eventdetails[2];
		$event = $eventdetails[3];
		$level = $eventdetails[4];
		$details = $eventdetails[5];

		$event_hash = &eventHash($node, $event, $details); 

		#define a hash index which uniquely identifies each event type for each event 
		# Lets try node_event_details!!
		$NMIS::eventTable{$event_hash}{current} = "true";
		$NMIS::eventTable{$event_hash}{startdate} = $eventdetails[0];
		$NMIS::eventTable{$event_hash}{lastchange} = $eventdetails[1];
		$NMIS::eventTable{$event_hash}{node} = $node;
		$NMIS::eventTable{$event_hash}{event} = $event;
		$NMIS::eventTable{$event_hash}{event_level} = $level;
		$NMIS::eventTable{$event_hash}{details} = $details;
		$NMIS::eventTable{$event_hash}{ack} = $eventdetails[6];
		$NMIS::eventTable{$event_hash}{escalate} = $eventdetails[7];
	}
	close(DATAFILE);
}

# This is different from the one in nodestats, it uses time as well to help sorting
sub loadEventStateTime {
	my @eventdetails;
	my ($time,$node,$event,$details,$event_hash,$level);

	undef %NMIS::eventTable;
	$NMIS::eventCount = 0;

	open(DATAFILE, "<$NMIS::config{event_file}") or warn returnTime." loadEventStateTime, Couldn't open file $NMIS::config{event_file}. $!\n";

	# File format is startdate,lastchange,node,event,event_level,ack,details
	while (<DATAFILE>) {
	        chomp;

		++$NMIS::eventCount;

		# whack all the splits into an array
		(@eventdetails) = split ",", $_;

		# using temp variables for readability
		$time = $eventdetails[0];
		$node = $eventdetails[2];
		$event = $eventdetails[3];
		$level = $eventdetails[4];
		$details = $eventdetails[5];

		$event_hash=&eventHashTime($time, $node, $event, $details); 

		#define a hash index which uniquely identifies each event type for each event 
		$NMIS::eventTable{$event_hash}{startdate} = $time;
		$NMIS::eventTable{$event_hash}{lastchange} = $eventdetails[1];
		$NMIS::eventTable{$event_hash}{node} = $node;
		$NMIS::eventTable{$event_hash}{event} = $event;
		$NMIS::eventTable{$event_hash}{event_level} = $level;
		$NMIS::eventTable{$event_hash}{details} = $details;
		$NMIS::eventTable{$event_hash}{ack} = $eventdetails[6];
		$NMIS::eventTable{$event_hash}{escalate} = $eventdetails[7];
	}
	close(DATAFILE);
}

sub eventHashTime {
	# Calculate the event hash the same way everytime.
 	#build an event hash string
	my $hash_time = shift;
	my $hash_node = shift;
	my $hash_event = shift;
	my $hash_details = shift;

	if ( $hash_event =~ /proactive/i ) { $hash_details = ""; }

	if ( $hash_details eq "" ) { return "$hash_time-$hash_node-$hash_event"; }
	else { 
		#Swap the spaces in $details for something else
		return "$hash_time-$hash_node-$hash_event-$hash_details"; 
	}
}

sub eventHash {
	# Calculate the event hash the same way everytime.
 	#build an event hash string
	my $hash_node = shift;
	my $hash_event = shift;
	my $hash_details = shift;
	
	if ( $hash_event =~ /proactive/i ) { $hash_details = ""; }

	if ( $hash_details eq "" ) { return "$hash_node-$hash_event"; }
	else { 
		return "$hash_node-$hash_event-$hash_details"; 
	}
}

sub eventExist {
	my $node = shift;
	my $event = shift;
	my $details = shift;

	my $event_hash = &eventHash($node,$event,$details);

	if ( $NMIS::eventTable{$event_hash}{node} eq $node ) {
		return "true";
	}
	else { 
		return "false"; 
	} 
}

sub checkEvent {	
	# Check event is called after determining that something is up!
	# Check event sees if an event for this node/interface exists 
	# if it exists it deletes it from the event state table/log
	# and then calls notify with the Up event including the time of the outage
        my %args = @_;
	my $outage;

	my $event_hash = &eventHash($args{node},$args{event},$args{details});

	# Load the evet State
	&loadEventState;
	if ( &eventExist($args{node},$args{event},$args{details}) eq "true" ) {
		# The opposite of this event exists log an UP and delete the event

		# the the event length for logging
		$outage = convertSecsHours(time - $NMIS::eventTable{$event_hash}{startdate});

		# Delete the event!!!!!
		$NMIS::eventTable{$event_hash}{current} = "false";
		&writeEventState;

		# Just log an up event now.
		if ( $args{event} eq "Node Down" ) {
			$args{event} = "Node Up";
			$args{details} = "Time=$outage secs";
		}
		elsif ( $args{event} eq "Interface Down" ) {
			$args{event} = "Interface Up";
			$args{details} = "$args{details} Time=$outage secs";
		}
		elsif ( $args{event} eq "SNMP Down" ) {
			$args{event} = "SNMP Up";
			$args{details} = "Time=$outage secs";
		}
		elsif ( $args{event} =~ /down/i ) {
			$args{event} =~ s/down/Up/i;
			if ( $args{details} eq "" ) { $args{details} = "Time=$outage secs"; }
			else { $args{details} = "$args{details} Time=$outage secs"; }
		}
		elsif ( $args{event} =~ /Proactive/ ) {
			$args{event} = "$args{event} Closed";
			$args{details} = "$args{details} Time=$outage secs";
		}
		&notify(node => $args{node}, role => $args{role}, type => $args{type}, event => $args{event}, level => $args{level}, details => $args{details});
	}
}

# Throw and Event to the event log
sub logEvent {	
	my $node = shift;
	my $event = shift;
	my $level = shift;
	my $details = shift;
	my $time = time;

	my $outage;

	if ( &outageCheck($node,time) eq "true" ) {
		$outage = &outageCheckHash($node,time);
		$details = "$details change=$NMIS::outageTable{$outage}{change}";
	}

	# Log the NEW event to the event log!!!
	open(OUTFILE, ">>$NMIS::config{event_log}") or warn returnTime." logEvent, Couldn't open file $NMIS::config{event_log}. $!\n";
	print OUTFILE "$time,$node,$event,$level,$details\n";
	close(OUTFILE); 
}

# Throw an Event to the event log
sub eventAdd {	
	my $node = shift;
	my $event = shift;
	my $level = shift;
	my $details = shift;
	my $time = time;

	my $valid = "false";
	my $event_hash = &eventHash($node,$event,$details);

	# before we log check the state table if there is currently an event outstanding.
	# Read the event file into the event State Hash
	&loadEventState;

	if ( &eventExist($node,$event,$details) eq "true" ) {
		# There exists an event of this type already.
		#Update the lastchange time with time now and write log outagain.
		$NMIS::eventTable{$event_hash}{lastchange} = time;
		&writeEventState;
	}
	# Otherwise it is a new event so better log it out to the file
	else {
		# We only put stateful type things in the state table
		# My ony concern here is that we have may get the event table 
		# loaded, someone changes the event file while we are doing 
		# stuff then we rewrite the event file hmmmm.... maybe write 
		# a routine which covers this off ie checking for changes to 
		# the file since we read it and handling it, i think the best 
		# thing to do is add event to hash and then manually add to 
		# the file.
		if (	( $event !~ /syslog|node reset|proactive|up/i ) 
		) {
			#insert a new entry in the event table.
			$NMIS::eventTable{$event_hash}{startdate} = time;
			$NMIS::eventTable{$event_hash}{lastchange} = time;
			$NMIS::eventTable{$event_hash}{node} = $node;
			$NMIS::eventTable{$event_hash}{event} = $event;
			$NMIS::eventTable{$event_hash}{event_level} = $level;
			$NMIS::eventTable{$event_hash}{details} = $details;
			$NMIS::eventTable{$event_hash}{ack} = "false";
			$NMIS::eventTable{$event_hash}{escalate} = 0;
			# Could implement autoacknowledge policy here!  Ie after hours
			# always ack!
			$valid = "true";
		}
		elsif (	( $event =~ /proactive/i ) 
		) {
			#insert a new entry in the event table.
			$NMIS::eventTable{$event_hash}{startdate} = time;
			$NMIS::eventTable{$event_hash}{lastchange} = time;
			$NMIS::eventTable{$event_hash}{node} = $node;
			$NMIS::eventTable{$event_hash}{event} = $event;
			$NMIS::eventTable{$event_hash}{event_level} = $level;
			$NMIS::eventTable{$event_hash}{details} = $details;
			$NMIS::eventTable{$event_hash}{ack} = "false";
			$NMIS::eventTable{$event_hash}{escalate} = 0;
			$valid = "true";
		}

		if ( $valid eq "true" ) {
			# Update the event file
			open(OUTFILE, ">>$NMIS::config{event_file}") or warn returnTime." eventAdd, Couldn't open file $NMIS::config{event_log}. $!\n";
			print OUTFILE "$time,$time,$node,$event,$level,$details,false,0\n";
			close(OUTFILE); 
		}
	}
} # eventAdd

sub eventAck {
        my %args = @_;
	my $event_hash;
	&loadEventState;
	#print STDERR "eventAck\tnode=$args{node} event=$args{event} details=$args{details} ack=$args{ack}\n";
	$event_hash = &eventHash($args{node}, $args{event}, $args{details}); 
	if ( $args{ack} eq "true" and $NMIS::eventTable{$event_hash}{ack} eq "false"  ) {
		$NMIS::eventTable{$event_hash}{ack} = "true";
		&logEvent("$args{node}", "$args{event}", "Normal", "$args{details} acknowledge=true");
		$NMIS::eventTable{$event_hash}{lastchange} = time;
	}
	elsif ( $args{ack} eq "false" and $NMIS::eventTable{$event_hash}{ack} eq "true"  ) {
		$NMIS::eventTable{$event_hash}{ack} = "false";
		&logEvent("$args{node}", "$args{event}", "$NMIS::eventTable{$event_hash}{event_level}", "$args{details} acknowledge=false");
		$NMIS::eventTable{$event_hash}{lastchange} = time;
	}
	&writeEventState;
} # eventAck

sub eventPolicy {
	## KS 16/4/01 event policy routine controls how events are handled
	# determines weather or not to do notify for given events.
	my %args = @_;
	my $event_key;
	my %event_result;
	my %event_table;
	if ( $args{role} eq "" ) { $args{role} = "access"; }
	if ( $args{type} eq "" ) { $args{type} = "router"; }
	
	if ( $args{event} =~ /Proactive/ ) {
		%event_table = loadCSV($NMIS::config{Events_Table},"Event:Level","\t");
		$event_key = lc($args{event}."_".$args{level});
	}
	else {
		%event_table = loadCSV($NMIS::config{Events_Table},$NMIS::config{Events_Key},"\t");
		$event_key = lc($args{event}."_".$args{role}."_".$args{type});
	}
	#send back results
	#level,notify,log,pager,mail
	$event_result{level} = $event_table{$event_key}{Level};
	$event_result{notify} = $event_table{$event_key}{Notify};
	$event_result{log} = $event_table{$event_key}{Log};
	$event_result{pager} = $event_table{$event_key}{Pager};
	$event_result{mail} = $event_table{$event_key}{Mail};
	if ($NMIS::debug) { print returnTime." eventPolicy argEvent=$args{event} argLevel=$args{level} event=$event_table{$event_key}{Event} level=$event_table{$event_key}{Level} notify=$event_table{$event_key}{Notify}\n"; }
	#print STDERR returnTime." eventPolicy key=$event_key role=$args{role} type=$args{type} argEvent=$args{event} argLevel=$args{level} event=$event_table{$event_key}{Event} level=$event_table{$event_key}{Level} notify=$event_table{$event_key}{Notify}\n";
	#Ahhh the magic of hash's
	return(%event_result);
} # eventPolicy

sub escalationPolicy {
	## KS 18/4/01 escalation policy routine controls how escalation is handled
	my %args = @_;
	my $esc_key;
	my %esc_table;
	my $level;
	my $target;
	
	$level = "Level".$args{level};

	%esc_table = loadCSV($NMIS::config{Escalation_Table},$NMIS::config{Escalation_Key},"\t");
	$esc_key = lc($args{group}."_".$args{role}."_".$args{type});
	
	#Ahhh the magic of hash's
	$target = $esc_table{$esc_key}{$level};
	if ( $target eq "" ) { 
		$args{group} = "default";
		$esc_key = lc($args{group}."_".$args{role}."_".$args{type});
		$target = $esc_table{$esc_key}{$level};
	}

	if ($NMIS::debug) { print returnTime." escalationPolicy target=$target level=$level type=$args{type} role=$args{role}\n"; }

	#send back results
	return($target);
} # escalationPolicy

sub notify {
	## KS 16/4/01 - Modified notify to implement event policy.
	# notify decides if/who/how anyone should be notified of an event.
	# and only notifies people once of an event by using the event table!
	my %args = @_;
	my $node = $args{node};
	my $role = $args{role};
	my $type = $args{type};
	my $event = $args{event};
	my $level = $args{level};
	my $details = $args{details};

	# Sets the default type and role for devices without entries in nodeTable
	if ( $role eq "" ) { $role = "access"; }
	if ( $type eq "" ) { $type = "router"; }

	my $outage;
	my $target;
	my $contact;
	my $message;
	my $notify = "false";
	my $mail = "false";
	my $pager = "false";
	my $log = "false";
	my $time = &returnDateStamp;
	my %event_result;
	my $pol_event;
	my $event_hash = &eventHash($node,$event,$details);
	
	# Before we notify etc check for duplicate events
	&loadEventState;
	if ( &eventExist($node,$event,$details) eq "false" ) {
		# Get the event policy and the rest is easy.
		if ( 	$event =~ /Proactive.*Closed/ ) { $pol_event = "Proactive Closed"; }
		elsif ( $event =~ /Proactive/ ) 	{ $pol_event = "Proactive"; }
		elsif ( $event =~ /down/i and $event !~ /SNMP|Node|Interface/ ) { 
			$pol_event = "Generic Down";
		}
		elsif ( $event =~ /up/i and $event !~ /SNMP|Node|Interface/ ) { 
			$pol_event = "Generic Up";
		}
		else 					{ $pol_event = $event; }
		%event_result = eventPolicy(event => $pol_event, role => $role, type => $type, level => $level);

		$notify = $event_result{notify};
		$mail = $event_result{mail};
		$pager = $event_result{pager};
		$level = $event_result{level};
		$log = $event_result{log};

		if ($NMIS::debug) { print returnTime." notify: node=$node event=$event role=$role level=$level details=$details\n"; }
		
		if ( $notify eq "" or $mail eq "" or $pager eq "" or $log eq "" ) { 
			$message = "Event type not defined: node=$node level=$level event=$event details=$details mail=$mail log=$log pager=$pager";
			logMessage("notify, $node, $message");
			if ($NMIS::debug) { print returnTime." notify $message\n"; }
		}
		if ( $log eq "true" and $event !~ /Proactive.*Closed/ ) {
			# Push the event onto the event table.
			&eventAdd("$node", "$event", "$level", "$details");
		}
	}
	else {
		if ( $NMIS::eventTable{$event_hash}{ack} eq "false" ) {
			$NMIS::eventTable{$event_hash}{lastchange} = time;
			&writeEventState;
		}
	}
	
	if ( &outageCheck($node,time) eq "true" ) {
		$notify = "false";
	}

	# Now based on the predetermined rules do the notification!!!!!!
	if ( $mail eq "true" and $notify eq "true" ) {
		## KS 1/4/2001 - set target to be whatever the sysContact email is.
		my %contact_table = loadCSV($NMIS::config{Contacts_Table},$NMIS::config{Contacts_Key},"\t");
		$contact = $NMIS::systemTable{sysContact};
		$target = $contact_table{'$contact'}{Email};
		# quick test to make sure the the target is never blank.
		if ( $target eq "" and $contact ne "default" ) {
			$target = $contact_table{default}{Email};
		}
		else { $target = $contact_table{default}{Email}; }
	    	if ($NMIS::debug) { print returnTime." notify: node=$node Contact=$contact and target=$target\n"; }
		&sendEmail(  	target => $target, 
				subject => "$time $args{node} $level $args{event} $args{details}", 
				body => "$time $args{node} $level $args{event} $args{details}\nhttp://$NMIS::config{nmis_host}$NMIS::config{nmis}?type=event&node=$args{node}"
		);
	}
	# Now if we had pager type notification here we would have 
	# pager=true set previously by the relevent types
	if ( $pager eq "true" and $notify eq "true" ) {
		#The method for email addresses using contacts could be used here to do paging.
		#&sendPagerMessage("$node", "$event", "$level", "$details");
	}

	if ( $log eq "true" ) {
		&logEvent("$node", "$event", "$level", "$details");
	}
} # end notify

## KS 18/4/01 - Added new sendEmail routine for use elsewhere.
sub sendEmail {
        my %args = @_;
	my $out;
	if ($NMIS::debug) { print returnTime." sendEmail target=$args{target} subject=$args{subject}\n"; }
	if  ( $args{target} ne "" ) {
		$out = `echo "$args{body}" | /usr/bin/mailx -s "$args{subject}" $args{target}`;
		if ( $out ne "" ) {
			print STDERR "Problem with &sendEmail\n";
			print STDERR "Output:\n$out";
		}
	}
	else {
		print STDERR "Problem with &sendEmail: target is BLANK\n";
	}
}

# I don't have a pager subsystem but here is where you call a script which 
# sends a page or sms or whatever
# TO BE WRITTEN
sub sendPagerMessage {
	my $node = shift;
	my $event = shift;
	my $level = shift;
	my $details = shift;
	my $time = time;
	print "YOU HAVE NOT CONFIGURED THE PAGER YET\n";
}


sub runEventDebug {
	&loadEventState;
	&writeEventState;
}

sub summaryStats {
	my $node = shift;
	my $type = shift;
	my $start = shift;
	my $end = shift;
	my $interface = shift;
	my $speed = shift;
	my @tmparray;
	my @tmpsplit;

	my $ERROR;
	my $database;
	my @splitret;
	my ($graphret,$xs,$ys);


	if ( $type eq "health" ) { 
		$database = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-reach.rrd";
		if ( -f $database ) {
			($graphret,$xs,$ys) = RRDs::graph "/dev/null", 
			"--start", "$start", 
			"--end", "$end", 
			"DEF:reach=$database:reachability:AVERAGE", 
			"DEF:avail=$database:availability:AVERAGE", 
			"DEF:health=$database:health:AVERAGE", 
			"DEF:response=$database:responsetime:AVERAGE", 
			"PRINT:reach:AVERAGE:reachable=%1.3lf",
			"PRINT:avail:AVERAGE:available=%1.3lf",
			"PRINT:health:AVERAGE:health=%1.3lf",
			"PRINT:response:AVERAGE:response=%1.2lf"
			;
		}
		else { logMessage("summaryStats, $node, no database file $database"); }
	}
	elsif ( $type eq "util" ) { 
		if ( $interface ne "" and  $speed eq "" ) {
			logMessage("summaryStats, $node, need speed to do interface stats interface=$interface");
		}
		my $tmpifDescr = &convertIfName($interface);
		$database = "$NMIS::config{database_root}/interface/$NMIS::systemTable{nodeType}/$node/$node-$tmpifDescr.rrd";
		if ( -f $database ) {
			($graphret,$xs,$ys) = RRDs::graph "/dev/null", 
			"--start", "$start", 
			"--end", "$end", 
			"DEF:input=$database:ifInOctets:AVERAGE", 
			"DEF:output=$database:ifOutOctets:AVERAGE", 
			"DEF:status=$database:ifOperStatus:AVERAGE", 
			"CDEF:inputUtil=input,8,*,$speed,/,100,*", 
			"CDEF:outputUtil=output,8,*,$speed,/,100,*", 
			"CDEF:totalUtil=outputUtil,inputUtil,+,2,/", 
			"PRINT:status:AVERAGE:availability=%1.2lf",
			"PRINT:inputUtil:AVERAGE:inputUtil=%1.2lf",
			"PRINT:outputUtil:AVERAGE:outputUtil=%1.2lf",
			"PRINT:totalUtil:AVERAGE:totalUtil=%1.2lf"
			;
		}
		else { logMessage("summaryStats, $node, no database file $database"); }
	}
	elsif ( $type eq "bits" ) { 
		my $tmpifDescr = &convertIfName($interface);
		$database = "$NMIS::config{database_root}/interface/$NMIS::systemTable{nodeType}/$node/$node-$tmpifDescr.rrd";
		if ( -f $database ) {
			($graphret,$xs,$ys) = RRDs::graph "/dev/null", 
			"--start", "$start", 
			"--end", "$end", 
			"DEF:input=$database:ifInOctets:AVERAGE", 
			"DEF:output=$database:ifOutOctets:AVERAGE", 
			"DEF:status=$database:ifOperStatus:AVERAGE", 
			"CDEF:inputBits=input,8,*", 
			"CDEF:outputBits=output,8,*", 
			"PRINT:status:AVERAGE:Availability=%1.2lf",
			"PRINT:inputBits:AVERAGE:inputBits=%1.2lf",
			"PRINT:outputBits:AVERAGE:outputBits=%1.2lf"
			;
		}
		else { logMessage("summaryStats, $node, no database file $database"); }
	}
	elsif ( $type eq "cpu" ) { 
		$database = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-health.rrd";
		if ( -f $database ) {
			($graphret,$xs,$ys) = RRDs::graph "/dev/null", 
			"--start", "$start", 
			"--end", "$end", 
			"DEF:avgBusy1=$database:avgBusy1:AVERAGE", 
			"DEF:avgBusy5=$database:avgBusy5:AVERAGE", 
			"DEF:MemUsed=$database:MemoryUsedPROC:AVERAGE", 
			"DEF:MemFree=$database:MemoryFreePROC:AVERAGE", 
			"CDEF:totalMem=MemUsed,MemFree,+",
			"CDEF:perUsedMem=MemUsed,totalMem,/,100,*",
			"CDEF:perFreeMem=MemFree,totalMem,/,100,*",
			"PRINT:avgBusy1:AVERAGE:avgBusy1min=%1.2lf",
			"PRINT:avgBusy5:AVERAGE:avgBusy5min=%1.2lf",
			"PRINT:perUsedMem:AVERAGE:ProcMemUsed=%1.2lf",
			"PRINT:perFreeMem:AVERAGE:ProcMemFree=%1.2lf",
			;
		}
		else { logMessage("summaryStats, $node, no database file $database"); }
	}
	else {
  		logMessage("summaryStats, $node not a valid option");
	}

	if ($ERROR = RRDs::error) { 
  		logMessage("summaryStats, $node, RRD graph error database=$database: $ERROR");
	} else {
		#print "GRAPH: node=$node, $NMIS::systemTable{nodeType}, $database $graphret\n";    
		#@splitret = split " ",$graphret;
		#return @splitret;
		return $graphret;
	}
}

## KS 9/4/01 added getGroupSummary for doing the metric stuff centrally!
## KS 24/5/01 fixed so that colors show for things which aren't complete
## also reweighted the metric to be reachability = %40, availability = %20
## and health = %40
sub getGroupSummary {
	my $group = shift;
	my $start_time = shift;
	my $end_time = shift;

	my @tmpsplit;
	my @tmparray;

	my %summaryHash;
	my $reportStats;
	my %nodecount;
	my $span;
	my $node;
	my $index;

	my (@devicelist,$i,@nodedetails);
	
	if ( $start_time eq "" ) { $start_time = "-8 hours"; }
	if ( $end_time eq "" ) { $end_time = time; }

	loadEventState;
	loadNodeDetails;
	
	# Insert some nice status info about the devices for the summary menu.
	foreach $node (sort ( keys (%NMIS::nodeTable) ) ) {
		#Only do the group
		if ( $group eq $NMIS::nodeTable{$node}{group} or $group eq "") {
			if ( eventExist($node,"Node Down","") eq "true" ) {
				($summaryHash{$node}{event_status},$summaryHash{$node}{event_color}) = eventLevel("Node Down",$NMIS::nodeTable{$node}{role});
			}
			else {
				($summaryHash{$node}{event_status},$summaryHash{$node}{event_color}) = eventLevel("Node Up",$NMIS::nodeTable{$node}{role});
			}
			
			loadSystemFile($node);
			$reportStats = summaryStats($node,"health",$start_time,$end_time);
			if ( $reportStats != 1 and $reportStats ne "" ) {
				@tmparray = split " ","@$reportStats";
				for ( $index = 0; $index <= $#tmparray; ++$index ) {
					@tmpsplit = split "=",@tmparray[$index];
					$summaryHash{$node}{$tmpsplit[0]} = $tmpsplit[1];
				}
				++$nodecount{total};
				if ( 	$summaryHash{$node}{reachable} !~ /NaN/	) {
					++$nodecount{reachable};
					$summaryHash{$node}{reachable_color} = colorHighGood($summaryHash{$node}{reachable});
					$summaryHash{total}{reachable} = $summaryHash{total}{reachable} + $summaryHash{$node}{reachable};
				}
				if ( $summaryHash{$node}{available} !~ /NaN/ ) {
					++$nodecount{available};
					$summaryHash{$node}{available_color} = colorHighGood($summaryHash{$node}{available});
					$summaryHash{total}{available} = $summaryHash{total}{available} + $summaryHash{$node}{available};
				}
				if ( $summaryHash{$node}{health} !~ /NaN/ ) {
					++$nodecount{health};
					$summaryHash{$node}{health_color} = colorHighGood($summaryHash{$node}{health});
					$summaryHash{total}{health} = $summaryHash{total}{health} + $summaryHash{$node}{health};
				}
				if ( $summaryHash{$node}{response} !~ /NaN/ ) {
					++$nodecount{response};
					$summaryHash{$node}{response_color} = colorResponseTime($summaryHash{$node}{response});
					$summaryHash{total}{response} = $summaryHash{total}{response} + $summaryHash{$node}{response};
				}
			}
			else {
				#print STDERR "getGroupSummary, pProblem with summarystats on node=$node stats=$reportStats\n";	
			}
			
			if ( $summaryHash{total}{reachable} > 0 ) {
				$summaryHash{average}{reachable} = sprintf("%.3f",$summaryHash{total}{reachable} / $nodecount{reachable} );
			}
			if ( $summaryHash{total}{available} > 0 ) {
				$summaryHash{average}{available} = sprintf("%.3f",$summaryHash{total}{available} / $nodecount{available} );
			}
			if ( $summaryHash{total}{health} > 0 ) {
				$summaryHash{average}{health} = sprintf("%.3f",$summaryHash{total}{health} / $nodecount{health} );
			}
			if ( $summaryHash{total}{response} > 0 ) {
				$summaryHash{average}{response} = sprintf("%.0f",$summaryHash{total}{response} / $nodecount{response} );
			}
			if ( $summaryHash{average}{reachable} == 0 ) { $summaryHash{average}{reachable} = 100; }
			if ( $summaryHash{average}{available} == 0 ) { $summaryHash{average}{reachable} = 100; }
			if ( $summaryHash{average}{health} == 0 ) { $summaryHash{average}{reachable} = 100; }

			if ( $summaryHash{total}{reachable} > 0 and $summaryHash{total}{available} > 0 and $summaryHash{total}{health} > 0 ) {
				# old weighting for metric
				#$summaryHash{average}{metric} = sprintf("%.3f",( 
				#	( $summaryHash{average}{reachable} ) +
				#	( $summaryHash{average}{available} ) +
				#	( $summaryHash{average}{health} ) ) / 3 
				#);
				# new weighting for metric
				$summaryHash{average}{metric} = sprintf("%.3f",( 
					( $summaryHash{average}{reachable} * 0.4 ) +
					( $summaryHash{average}{available} * 0.2 ) +
					( $summaryHash{average}{health} ) * 0.4 )
				);
			}
			$summaryHash{average}{reachable_color} = colorHighGood($summaryHash{average}{reachable});
			$summaryHash{average}{available_color} = colorHighGood($summaryHash{average}{available});
			$summaryHash{average}{health_color} = colorHighGood($summaryHash{average}{health});
			$summaryHash{average}{response_color} = colorResponseTime($summaryHash{average}{response});
			$summaryHash{average}{metric_color} = colorHighGood($summaryHash{average}{metric});
			$summaryHash{average}{count} = $nodecount{total};
		}
	}

	return %summaryHash;
} # end getGroupSummary

sub colorHighGood {
	my $threshold = shift;
	my $color = "";

	if ( ( $threshold =~ /^[a-zA-Z]/ ) || ( $threshold eq "") )  { $color = "#FFFFFF"; }
	elsif ( $threshold eq "N/A" )  { $color = "#FFFFFF"; }
	elsif ( $threshold >= 100 ) { $color = "#00FF00"; }
	elsif ( $threshold >= 95 ) { $color = "#00EE00"; }
	elsif ( $threshold >= 90 ) { $color = "#00DD00"; }
	elsif ( $threshold >= 85 ) { $color = "#00CC00"; }
	elsif ( $threshold >= 80 ) { $color = "#00BB00"; }
	elsif ( $threshold >= 75 ) { $color = "#00AA00"; }
	elsif ( $threshold >= 70 ) { $color = "#009900"; }
	elsif ( $threshold >= 65 ) { $color = "#008800"; }
	elsif ( $threshold >= 60 ) { $color = "#FFFF00"; }
	elsif ( $threshold >= 55 ) { $color = "#FFEE00"; }
	elsif ( $threshold >= 50 ) { $color = "#FFDD00"; }
	elsif ( $threshold >= 45 ) { $color = "#FFCC00"; }
	elsif ( $threshold >= 40 ) { $color = "#FFBB00"; }
	elsif ( $threshold >= 35 ) { $color = "#FFAA00"; }
	elsif ( $threshold >= 30 ) { $color = "#FF9900"; }
	elsif ( $threshold >= 25 ) { $color = "#FF8800"; }
	elsif ( $threshold >= 20 ) { $color = "#FF7700"; }
	elsif ( $threshold >= 15 ) { $color = "#FF6600"; }
	elsif ( $threshold >= 10 ) { $color = "#FF5500"; }
	elsif ( $threshold >= 5 )  { $color = "#FF3300"; }
	elsif ( $threshold > 0 )   { $color = "#FF1100"; }
	elsif ( $threshold == 0 )  { $color = "#FF0000"; }

	return $color;
}

sub colorPort {
	my $threshold = shift;
	my $color = "";

	if ( $threshold >= 60 ) { $color = "#FFFF00"; }
	elsif ( $threshold < 60 ) { $color = "#00FF00"; }

	return $color;
}

sub colorLowGood {
	my $threshold = shift;
	my $color = "";

	if ( ( $threshold =~ /^[a-zA-Z]/ ) || ( $threshold eq "") )  { $color = "#FFFFFF"; }
	elsif ( $threshold == 0 ) { $color = "#00FF00"; }
	elsif ( $threshold <= 5 ) { $color = "#00EE00"; }
	elsif ( $threshold <= 10 ) { $color = "#00DD00"; }
	elsif ( $threshold <= 15 ) { $color = "#00CC00"; }
	elsif ( $threshold <= 20 ) { $color = "#00BB00"; }
	elsif ( $threshold <= 25 ) { $color = "#00AA00"; }
	elsif ( $threshold <= 30 ) { $color = "#009900"; }
	elsif ( $threshold <= 35 ) { $color = "#008800"; }
	elsif ( $threshold <= 40 ) { $color = "#FFFF00"; }
	elsif ( $threshold <= 45 ) { $color = "#FFEE00"; }
	elsif ( $threshold <= 50 ) { $color = "#FFDD00"; }
	elsif ( $threshold <= 55 ) { $color = "#FFCC00"; }
	elsif ( $threshold <= 60 ) { $color = "#FFBB00"; }
	elsif ( $threshold <= 65 ) { $color = "#FFAA00"; }
	elsif ( $threshold <= 70 ) { $color = "#FF9900"; }
	elsif ( $threshold <= 75 ) { $color = "#FF8800"; }
	elsif ( $threshold <= 80 ) { $color = "#FF7700"; }
	elsif ( $threshold <= 85 ) { $color = "#FF6600"; }
	elsif ( $threshold <= 90 ) { $color = "#FF5500"; }
	elsif ( $threshold <= 95 ) { $color = "#FF4400"; }
	elsif ( $threshold < 100 ) { $color = "#FF3300"; }
	elsif ( $threshold == 100 )  { $color = "#FF1100"; }
	elsif ( $threshold <= 110 )  { $color = "#FF0055"; }
	elsif ( $threshold <= 120 )  { $color = "#FF0066"; }
	elsif ( $threshold <= 130 )  { $color = "#FF0077"; }
	elsif ( $threshold <= 140 )  { $color = "#FF0088"; }
	elsif ( $threshold <= 150 )  { $color = "#FF0099"; }
	elsif ( $threshold <= 160 )  { $color = "#FF00AA"; }
	elsif ( $threshold <= 170 )  { $color = "#FF00BB"; }
	elsif ( $threshold <= 180 )  { $color = "#FF00CC"; }
	elsif ( $threshold <= 190 )  { $color = "#FF00DD"; }
	elsif ( $threshold <= 200 )  { $color = "#FF00EE"; }
	elsif ( $threshold > 200 )  { $color = "#FF00FF"; }

	return $color;
}

sub colorResponseTime {
	my $threshold = shift;
	my $color = "";

	if ( ( $threshold =~ /^[a-zA-Z]/ ) || ( $threshold eq "") )  { $color = "#FFFFFF"; }
	elsif ( $threshold <= 1 ) { $color = "#00FF00"; }
	elsif ( $threshold <= 20 ) { $color = "#00EE00"; }
	elsif ( $threshold <= 50 ) { $color = "#00DD00"; }
	elsif ( $threshold <= 100 ) { $color = "#00CC00"; }
	elsif ( $threshold <= 200 ) { $color = "#00BB00"; }
	elsif ( $threshold <= 250 ) { $color = "#00AA00"; }
	elsif ( $threshold <= 300 ) { $color = "#009900"; }
	elsif ( $threshold <= 350 ) { $color = "#FFFF00"; }
	elsif ( $threshold <= 400 ) { $color = "#FFEE00"; }
	elsif ( $threshold <= 450 ) { $color = "#FFDD00"; }
	elsif ( $threshold <= 500 ) { $color = "#FFCC00"; }
	elsif ( $threshold <= 550 ) { $color = "#FFBB00"; }
	elsif ( $threshold <= 600 ) { $color = "#FFAA00"; }
	elsif ( $threshold <= 650 ) { $color = "#FF9900"; }
	elsif ( $threshold <= 700 ) { $color = "#FF8800"; }
	elsif ( $threshold <= 750 ) { $color = "#FF7700"; }
	elsif ( $threshold <= 800 ) { $color = "#FF6600"; }
	elsif ( $threshold <= 850 ) { $color = "#FF5500"; }
	elsif ( $threshold <= 900 ) { $color = "#FF4400"; }
	elsif ( $threshold <= 950 )  { $color = "#FF3300"; }
	elsif ( $threshold < 1000 )   { $color = "#FF1100"; }
	elsif ( $threshold > 1000 )  { $color = "#FF0000"; }

	return $color;
}

sub thresholdResponse {
	my $node = shift;
	my $role = shift;
	my $threshold = shift;

	my $level;

	if ( $threshold <= 500 ) { $level = "1"; }
	elsif ( $threshold <= 750 ) { $level = "2"; }
	elsif ( $threshold <= 1000 ) { $level = "3"; }
	elsif ( $threshold <= 1500 )   { $level = "4"; }
	elsif ( $threshold > 1500 )  { $level = "5"; }

	if ( $role eq "core" ) { $level + 2; }
	elsif ( $role eq "distribution" ) { $level + 1; }
	
	$level = &eventNumberLevel($level);

	return $level;
}

sub thresholdLowPercent {
	my $node = shift;
	my $role = shift;
	my $threshold = shift;

	my $level;

	if ( $threshold <= 30 ) { $level = "1"; }
	elsif ( $threshold <= 50 ) { $level = "2"; }
	elsif ( $threshold <= 75 ) { $level = "3"; }
	elsif ( $threshold <= 90 )   { $level = "4"; }
	elsif ( $threshold <= 95 )  { $level = "5"; }
	elsif ( $threshold <= 100 )  { $level = "6"; }

	if ( $level == 1 ) { $level = 1; }
	elsif ( $role eq "core" ) { $level = $level + 2; }
	elsif ( $role eq "distribution" ) { $level = $level + 1; }
	
	$level = &eventNumberLevel($level);

	return $level;
}

sub thresholdHighPercent {
	my $node = shift;
	my $role = shift;
	my $threshold = shift;

	my $level;

	if ( $threshold == 100 ) { $level = "1"; }
	elsif ( $threshold >= 95 ) { $level = "2"; }
	elsif ( $threshold >= 90 ) { $level = "3"; }
	elsif ( $threshold >= 75 )   { $level = "4"; }
	elsif ( $threshold >= 50 )  { $level = "5"; }
	elsif ( $threshold < 50 )  { $level = "6"; }

	if ( $level == 1 ) { $level = 1; }
	elsif ( $role eq "core" ) { $level = $level + 2; }
	elsif ( $role eq "distribution" ) { $level = $level + 1; }
	
	$level = &eventNumberLevel($level);

	return $level;
}

sub thresholdHighPercentLoose {
	my $node = shift;
	my $role = shift;
	my $threshold = shift;

	my $level;

	if ( $threshold >= 85 ) { $level = "1"; }
	elsif ( $threshold >= 70 ) { $level = "2"; }
	elsif ( $threshold >= 60 ) { $level = "3"; }
	elsif ( $threshold >= 50 )   { $level = "4"; }
	elsif ( $threshold >= 40 )  { $level = "5"; }
	elsif ( $threshold < 40 )  { $level = "6"; }

	if ( $level == 1 ) { $level = 1; }
	elsif ( $role eq "core" ) { $level = $level + 2; }
	elsif ( $role eq "distribution" ) { $level = $level + 1; }
	
	$level = &eventNumberLevel($level);

	return $level;
}

sub thresholdMemory {
	my $node = shift;
	my $role = shift;
	my $threshold = shift;

	my $level;

	if    ( $threshold >= 40 ) { $level = "1"; }
	elsif ( $threshold >= 30 ) { $level = "2"; }
	elsif ( $threshold >= 20 ) { $level = "3"; }
	elsif ( $threshold >= 15 ) { $level = "4"; }
	elsif ( $threshold >= 10 ) { $level = "5"; }
	elsif ( $threshold >= 0 ) { $level = "6"; }

	if ( $level == 1 ) { $level = 1; }
	elsif ( $role eq "distribution" ) { $level = $level + 1; }
	elsif ( $role eq "core" ) { $level = $level + 2; }
	
	$level = &eventNumberLevel($level);

	return $level;
}

sub thresholdInterfaceUtil {
	my $node = shift;
	my $role = shift;
	my $threshold = shift;

	my $level;

	# for testing the thresholding! don't get much utilisation here!
	#if    ( $threshold == 0 ) { $level = "1"; }
	#elsif ( $threshold > 0 ) { $level = "4"; }
	if ( $threshold < 60 ) { $level = "1"; }
	elsif ( $threshold >= 60 ) { $level = "2"; }
	elsif ( $threshold >= 70 ) { $level = "3"; }
	elsif ( $threshold >= 80 ) { $level = "4"; }
	elsif ( $threshold >= 90 ) { $level = "5"; }
	elsif ( $threshold >= 95 ) { $level = "6"; }

	if ( $level == 1 ) { $level = 1; }
	elsif ( $role eq "distribution" ) { $level = $level + 1; }
	elsif ( $role eq "core" ) { $level = $level + 2; }
	
	$level = &eventNumberLevel($level);

	return $level;
}

sub thresholdInterfaceAvail {
	my $node = shift;
	my $role = shift;
	my $threshold = shift;

	my $level;
	
	## KS 12/4/01 - Tweaked the thresholds a bit, little less dramatic.
	if    ( $threshold >= 99 ) { $level = "1"; }
	elsif ( $threshold >= 95 ) { $level = "1"; }
	elsif ( $threshold >= 80 ) { $level = "2"; }
	elsif ( $threshold >= 70 ) { $level = "3"; }
	elsif ( $threshold >= 60 ) { $level = "4"; }
	elsif ( $threshold >= 50 ) { $level = "5"; }
	elsif ( $threshold < 50 ) { $level = "6"; }

	if ( $level == 1 ) { $level = 1; }
	elsif ( $role eq "distribution" ) { $level = $level + 1; }
	elsif ( $role eq "core" ) { $level = $level + 2; }
	
	$level = &eventNumberLevel($level);

	return $level;
}

sub eventLevel {
	my $event = shift;
	my $role = shift;

	my $event_level;
	my $event_color;

	# Convert DOWN to Node Down
	$event =~ s/^down/Node Down/gi;
	
	if ( $event eq "Node Down" ) {
	 	if ( $role eq "core" ) { $event_level = "Critical"; }
	 	elsif ( $role eq "distribution" ) { $event_level = "Major"; }
	 	elsif ( $role eq "access" ) { $event_level = "Minor"; }
	}
	elsif ( $event eq "Interface Down" ) {
	 	if ( $role eq "core" ) { $event_level = "Major"; }
	 	elsif ( $role eq "distribution" ) { $event_level = "Minor"; }
	 	elsif ( $role eq "access" ) { $event_level = "Warning";	}
	}
	elsif ( $event =~ /up/i ) { $event_level = "Normal"; }
	else { $event_level = "Unknown"; }
	$event_color = &eventColor($event_level);
	return ($event_level,$event_color);
} # eventLevel

sub overallNodeStatus {
	my $netType = shift;
	my $roleType = shift;
	
	my $node;
	my $event_status;
	my $overall_status;
	my $status_number;
	my $total_status;
	my $multiplier;
	my $group;
	my $status;

	my %statusHash;

	#print STDERR &returnDateStamp." overallNodeStatus: netType=$netType roleType=$roleType\n";
	&loadEventState;

	if ( $netType eq "" and $roleType eq "" ) {
		foreach $node (sort ( keys (%NMIS::nodeTable) ) ) {
			if ( &eventExist($node,"Node Down","") eq "true" ) {
				($event_status) = &eventLevel("Node Down",$NMIS::nodeTable{$node}{role});
			}
			else {
				($event_status) = &eventLevel("Node Up",$NMIS::nodeTable{$node}{role});
			}
			++$statusHash{$event_status};
			++$statusHash{count};
		}	
	}
	elsif ( $netType ne "" and $roleType ne "" ) {
	        foreach $node (sort ( keys (%NMIS::nodeTable) ) ) {
			if (	( $NMIS::nodeTable{$node}{net} eq "$netType" &&
				$NMIS::nodeTable{$node}{role} eq "$roleType" )
				
			) {
				if ( &eventExist($node,"Node Down","") eq "true" ) {
					($event_status) = &eventLevel("Node Down",$NMIS::nodeTable{$node}{role});
				}
				else {
					($event_status) = &eventLevel("Node Up",$NMIS::nodeTable{$node}{role});
				}
				++$statusHash{$event_status};
				++$statusHash{count};
			}
	        }
        }
	elsif ( $netType ne "" and $roleType eq "" ) {
		$group = $netType;
		foreach $node (sort ( keys (%NMIS::nodeTable) ) ) {
			if ( $NMIS::nodeTable{$node}{group} eq $group ) {
				if ( &eventExist($node,"Node Down","") eq "true" ) {
					($event_status) = &eventLevel("Node Down",$NMIS::nodeTable{$node}{role});
				}
				else {
					($event_status) = &eventLevel("Node Up",$NMIS::nodeTable{$node}{role});
				}
				++$statusHash{$event_status};
				++$statusHash{count};
				#print STDERR &returnDateStamp." overallNodeStatus: $node $group $event_status event=$statusHash{$event_status} count=$statusHash{count}\n";
			}
		}
	}

	$status_number = 100 * $statusHash{Normal};
	$status_number = $status_number + ( 90 * $statusHash{Warning} );
	$status_number = $status_number + ( 75 * $statusHash{Minor} );
	$status_number = $status_number + ( 60 * $statusHash{Major} );
	$status_number = $status_number + ( 50 * $statusHash{Critical} );
	$status_number = $status_number + ( 40 * $statusHash{Fatal} );
	if ( $status_number != 0 and $statusHash{count} != 0 ) {
		$status_number = $status_number / $statusHash{count};
	}
	#print STDERR "New CALC: status_number=$status_number count=$statusHash{count}\n";

	## KS 11/4/01 - Fixed up status for single node groups.
	# if the node count is one we do not require weighting.
	if ( $statusHash{count} == 1 ) {
		delete ($statusHash{count});
		foreach $status (keys %statusHash) {
			if ( $statusHash{$status} ne "" and $statusHash{$status} ne "count" ) {
				$overall_status = $status;
				#print STDERR returnDateStamp." overallNodeStatus netType=$netType status=$status hash=$statusHash{$status}\n";
			}
		}
	}
	elsif ( $status_number != 0  ) {
		if ( $status_number == 100 ) { $overall_status = "Normal"; }
		elsif ( $status_number >= 95 ) { $overall_status = "Warning"; }
		elsif ( $status_number >= 90 ) { $overall_status = "Minor"; }
		elsif ( $status_number >= 70 ) { $overall_status = "Major"; }
		elsif ( $status_number >= 50 ) { $overall_status = "Critical"; }
		elsif ( $status_number < 50 ) { $overall_status = "Fatal"; }
	}
	else {
		$overall_status = "Unknown";
	}

	return $overall_status;
} # end overallNodeStatus

sub logMessage {
	my $string = shift;
	open(DATA, ">>$NMIS::config{nmis_log}") or warn returnTime." logMessage, Couldn't not open log file $NMIS::config{nmis_log}. $!\n";
	print DATA &returnDateStamp.",$string\n";
	close(DATA);
} # end logMessage

sub createInterfaceInfo {
	my $node;
	my $index;
	my $tmpDesc;
	my $intHash;
	my $interface;
	my %interfaceInfo;
	my %interfaceTable;
	
	&loadNodeDetails;

	if ($NMIS::debug) { print returnTime." Getting Interface Info from all nodes.\n"; }

	# Write a node entry for each node
	foreach $node (sort( keys(%NMIS::nodeTable) ) )  {

		if ( $NMIS::nodeTable{$node}{collect} eq "true" ) {
			if ($NMIS::debug>7) { print "\tgetting $node interface information\n"; }
			if ( -f "$NMIS::config{'<nmis_var>'}/$node-interface.dat" ) {
				%interfaceTable = &loadCSV("$NMIS::config{'<nmis_var>'}/$node-interface.dat","interface","\t");
				foreach $interface (keys %interfaceTable) {
		
					$tmpDesc = &convertIfName($interfaceTable{$interface}{ifDescr});
		
					$intHash = "$node-$tmpDesc";
					
					if ($NMIS::debug==9) { print "\t\t$node $tmpDesc hash=$intHash $interfaceTable{$interface}{ifDescr}\n"; }
		
					if ( $interfaceTable{$interface}{ifDescr} ne "" ) { 
			  			$interfaceInfo{$intHash}{node} = $node;
			  			$interfaceInfo{$intHash}{ifIndex} = $interfaceTable{$interface}{ifIndex};
						$interfaceInfo{$intHash}{ifDescr} = $interfaceTable{$interface}{ifDescr};
						$interfaceInfo{$intHash}{collect} = $interfaceTable{$interface}{collect};
						$interfaceInfo{$intHash}{ifType} = $interfaceTable{$interface}{ifType};
						$interfaceInfo{$intHash}{ifSpeed} = $interfaceTable{$interface}{ifSpeed};
						$interfaceInfo{$intHash}{ifAdminStatus} = $interfaceTable{$interface}{ifAdminStatus};
						$interfaceInfo{$intHash}{ifOperStatus} = $interfaceTable{$interface}{ifOperStatus};
						$interfaceInfo{$intHash}{ifLastChange} = $interfaceTable{$interface}{ifLastChange};
						$interfaceInfo{$intHash}{Description} = $interfaceTable{$interface}{Description};
						$interfaceInfo{$intHash}{portModuleIndex} = $interfaceTable{$interface}{portModuleIndex};
						$interfaceInfo{$intHash}{portIndex} = $interfaceTable{$interface}{portIndex};
						$interfaceInfo{$intHash}{portDuplex} = $interfaceTable{$interface}{portDuplex};
						$interfaceInfo{$intHash}{portIfIndex} = $interfaceTable{$interface}{portIfIndex};
						$interfaceInfo{$intHash}{portSpantreeFastStart} = $interfaceTable{$interface}{portSpantreeFastStart};
						$interfaceInfo{$intHash}{vlanPortVlan} = $interfaceTable{$interface}{vlanPortVlan};
						$interfaceInfo{$intHash}{portAdminSpeed} = $interfaceTable{$interface}{portAdminSpeed};
						$interfaceInfo{$intHash}{ipAdEntAddr} = $interfaceTable{$interface}{ipAdEntAddr};
						$interfaceInfo{$intHash}{ipAdEntNetMask} = $interfaceTable{$interface}{ipAdEntNetMask};
						$interfaceInfo{$intHash}{ipSubnet} = $interfaceTable{$interface}{ipSubnet};
						$interfaceInfo{$intHash}{ipSubnetBits} = $interfaceTable{$interface}{ipSubnetBits};
					}
				}
			}
		}
	} # foreach $linkname
	# Write the interface table out.
	if ($NMIS::debug) { print returnTime." Writing Interface Info from all nodes.\n"; }
	&writeCSV(%interfaceInfo,$NMIS::config{Interface_Table},"\t");
}

sub loadInterfaceInfo {
	%NMIS::interfaceInfo = &loadCSV($NMIS::config{Interface_Table},$NMIS::config{Interface_Key},"\t");
}

sub outageAdd {
	# Outage add inserts an entry into the outage table 
	# Start must be before "now" and Change Number must equal to something.

	my $node = shift;
	my $start = shift;
	my $end = shift;
	my $change = shift;
	my $error = "false";

	# Convert nasty adhoc date entry into seconds
	$start = parsedate("$start");
	$end = parsedate("$end");

	# Now - 5 minutes
	if ( $start < time - (60 * 5) ) {
		$error = "Cannot add Planned Outage with time less than \"now\" ".&returnDateStamp.".";
	}
	if ( $start >= $end ) {
		$error = "Cannot add start time less then or equal to end time.";
	}
	if ( $change eq "" ) {
		$error = "Change number must be included with changes.";
	}
	if ( $node eq "" ) {
		$error = "Node name must be provided.";
	}

	if ( $error eq "false" ) {
		open(DATAFILE, ">>$NMIS::config{outage_file}") or warn returnTime." outageAdd, Couldn't open interface file $NMIS::config{outage_file}. $!\n";
		print DATAFILE "$node,";
		print DATAFILE "$start,";
		print DATAFILE "$end,";
		print DATAFILE "$change";
		print DATAFILE "\n";
		close(DATAFILE);
	}
	return $error;
}

sub outageDelete {

	my $node = shift;
	my $start = shift;
	my $end = shift;
	my $change = shift;

	my $outage;
	my $outageHash;

	&outageLoad;
	$outageHash = "$node-$start-$end-$change";

	open(DATAFILE, ">$NMIS::config{outage_file}") or warn returnTime." outageDelete, Couldn't open Outage file $NMIS::config{outage_file}. $!\n";
       	foreach $outage (keys(%NMIS::outageTable)) {
		# if is doesn't match write to the file
		if ( $outage ne $outageHash and $NMIS::outageTable{$outage}{node} ne "" ) {
			print DATAFILE "$NMIS::outageTable{$outage}{node},";
			print DATAFILE "$NMIS::outageTable{$outage}{start},";
			print DATAFILE "$NMIS::outageTable{$outage}{end},";
			print DATAFILE "$NMIS::outageTable{$outage}{change}";
			print DATAFILE "\n";
		}
		# it matches so zero the hash for that value
		else {
		 	#$NMIS::outageTable{$outage} = "";
		 	$NMIS::outageTable{$outage}{node} = "";
		}


	}
	close (DATAFILE);
}

sub outageLoad {
	
	my $outage;
  	my @entry;

	open(DATAFILE, "$NMIS::config{outage_file}") or warn returnTime." outageLoad, Cannot open Outage file $NMIS::config{outage_file}. $!\n";

	while (<DATAFILE>) {
	chomp;
	# Does the line from configfile have a comment?
		if ( $_ !~ /^#|^;/) {
			@entry = split(",", $_);
			
			$outage = "$entry[0]-$entry[1]-$entry[2]-$entry[3]";
			
			$NMIS::outageTable{$outage}{node} = $entry[0];
			$NMIS::outageTable{$outage}{start} = $entry[1];
			$NMIS::outageTable{$outage}{end} = $entry[2];
			$NMIS::outageTable{$outage}{change} = $entry[3];
		}
	}
	close(DATAFILE);
}

sub outageCheck {
	my $node = shift;
	my $time = shift;

	my $outageCurrent = "false";
	my $outage;

	&outageLoad;

	# Get each of the nodes info in a HASH for playing with
       	foreach $outage (keys(%NMIS::outageTable)) {
		if ( 	$node eq $NMIS::outageTable{$outage}{node} and 
			$time >= $NMIS::outageTable{$outage}{start} and 
			$time <= $NMIS::outageTable{$outage}{end} and
			# Seems blanks match scheduled outage stuff lets make sure they aren't blank too.
			$NMIS::outageTable{$outage}{start} > 9000 and 
			$NMIS::outageTable{$outage}{END} > 9000 
		) {
			$outageCurrent = "true";
		}
		elsif ( $node eq $NMIS::outageTable{$outage}{node} and 
			$time < $NMIS::outageTable{$outage}{start} 
		) {
			$outageCurrent = "pending";
		}
		elsif ( $node eq $NMIS::outageTable{$outage}{node} and 
			$time > $NMIS::outageTable{$outage}{end} 
		) {
			$outageCurrent = "closed";
		}
	}

	return "$outageCurrent";
}

sub outageCheckHash {
	# This function returns the hash key of an outage if an outage exists for it.
	my $node = shift;
	my $time = shift;

	my $outageCurrent = "false";
	my $outage;

	&outageLoad;

	# Get each of the nodes info in a HASH for playing with
       	foreach $outage (keys(%NMIS::outageTable)) {
		if ( 	$node eq $NMIS::outageTable{$outage}{node} and 
			$time >= $NMIS::outageTable{$outage}{start} and 
			$time <= $NMIS::outageTable{$outage}{end} 
		) {
			$outageCurrent = $outage;
		}
	}

	return "$outageCurrent";
}

sub convertTime {
	my $time = shift;
	my $timenow = time;
	my $newtime;

	# convert length code into Graph start time
	if ( $time eq "2hours" ) { $newtime = $timenow - 2 * 60 * 60; }
	elsif ( $time eq "3hours" ) { $newtime = $timenow - 3 * 60 * 60; }
	elsif ( $time eq "4hours" ) { $newtime = $timenow - 4 * 60 * 60; }
	elsif ( $time eq "5hours" ) { $newtime = $timenow - 5 * 60 * 60; }
	elsif ( $time eq "6hours" ) { $newtime = $timenow - 6 * 60 * 60; }
	elsif ( $time eq "7hours" ) { $newtime = $timenow - 7 * 60 * 60; }
	elsif ( $time eq "8hours" ) { $newtime = $timenow - 8 * 60 * 60; }
	elsif ( $time eq "10hours" ) { $newtime = $timenow - 10 * 60 * 60; }
	elsif ( $time eq "12hours" ) { $newtime = $timenow - 12 * 60 * 60; }
	elsif ( $time eq "16hours" ) { $newtime = $timenow - 16 * 60 * 60; }
	elsif ( $time eq "18hours" ) { $newtime = $timenow - 18 * 60 * 60; }
	elsif ( $time eq "20hours" ) { $newtime = $timenow - 20 * 60 * 60; }
	elsif ( $time eq "36hours" ) { $newtime = $timenow - 36 * 60 * 60; }
	elsif ( $time eq "1day" ) { $newtime = $timenow - 1 * 24 * 60 * 60; }
	elsif ( $time eq "2days" ) { $newtime = $timenow - 2 * 24 * 60 * 60; }
	elsif ( $time eq "3days" ) { $newtime = $timenow - 3 * 24 * 60 * 60; }
	elsif ( $time eq "4days" ) { $newtime = $timenow - 4 * 24 * 60 * 60; }
	elsif ( $time eq "5days" ) { $newtime = $timenow - 5 * 24 * 60 * 60; }
	elsif ( $time eq "6days" ) { $newtime = $timenow - 6 * 24 * 60 * 60; }
	elsif ( $time eq "1week" ) { $newtime = $timenow - 1 * 7 * 24 * 60 * 60; }
	elsif ( $time eq "2weeks" ) { $newtime = $timenow - 2 * 7 * 24 * 60 * 60; }
	elsif ( $time eq "3weeks" ) { $newtime = $timenow - 3 * 7 * 24 * 60 * 60; }
	elsif ( $time eq "4weeks" ) { $newtime = $timenow - 4 * 7 * 24 * 60 * 60; }
	elsif ( $time eq "5weeks" ) { $newtime = $timenow - 5 * 7 * 24 * 60 * 60; }
	elsif ( $time eq "6weeks" ) { $newtime = $timenow - 6 * 7 * 24 * 60 * 60; }
	elsif ( $time eq "7weeks" ) { $newtime = $timenow - 7 * 7 * 24 * 60 * 60; }
	elsif ( $time eq "8weeks" ) { $newtime = $timenow - 8 * 7 * 24 * 60 * 60; }
	elsif ( $time eq "1month" ) { $newtime = $timenow - 1 * 31 * 24 * 60 * 60; }
	elsif ( $time eq "2months" ) { $newtime = $timenow - 2 * 31 * 24 * 60 * 60; }
	elsif ( $time eq "4months" ) { $newtime = $timenow - 4 * 31 * 24 * 60 * 60; }
	elsif ( $time eq "6months" ) { $newtime = $timenow - 6 * 31 * 24 * 60 * 60; }
	elsif ( $time eq "1year" ) { $newtime= $timenow - 365 * 24 * 60 * 60; }

	return $newtime;
}

sub convertTimeLength {
	my $time = shift;
	my $newtime;

	# convert length code into Graph start time
	if ( $time eq "2hours" ) { $newtime = 2 * 60 * 60; }
	elsif ( $time eq "3hours" ) { $newtime = 3 * 60 * 60; }
	elsif ( $time eq "4hours" ) { $newtime = 4 * 60 * 60; }
	elsif ( $time eq "5hours" ) { $newtime = 5 * 60 * 60; }
	elsif ( $time eq "6hours" ) { $newtime = 6 * 60 * 60; }
	elsif ( $time eq "7hours" ) { $newtime = 7 * 60 * 60; }
	elsif ( $time eq "8hours" ) { $newtime = 8 * 60 * 60; }
	elsif ( $time eq "10hours" ) { $newtime = 10 * 60 * 60; }
	elsif ( $time eq "12hours" ) { $newtime = 12 * 60 * 60; }
	elsif ( $time eq "16hours" ) { $newtime = 16 * 60 * 60; }
	elsif ( $time eq "18hours" ) { $newtime = 18 * 60 * 60; }
	elsif ( $time eq "20hours" ) { $newtime = 20 * 60 * 60; }
	elsif ( $time eq "24hours" ) { $newtime = 24 * 60 * 60; }
	elsif ( $time eq "36hours" ) { $newtime = 36 * 60 * 60; }
	elsif ( $time eq "1day" ) { $newtime = 1 * 24 * 60 * 60; }
	elsif ( $time eq "2days" ) { $newtime = 2 * 24 * 60 * 60; }
	elsif ( $time eq "3days" ) { $newtime = 3 * 24 * 60 * 60; }
	elsif ( $time eq "4days" ) { $newtime = 4 * 24 * 60 * 60; }
	elsif ( $time eq "5days" ) { $newtime = 5 * 24 * 60 * 60; }
	elsif ( $time eq "6days" ) { $newtime = 6 * 24 * 60 * 60; }
	elsif ( $time eq "1week" ) { $newtime = 1 * 7 * 24 * 60 * 60; }
	elsif ( $time eq "2weeks" ) { $newtime = 2 * 7 * 24 * 60 * 60; }
	elsif ( $time eq "3weeks" ) { $newtime = 3 * 7 * 24 * 60 * 60; }
	elsif ( $time eq "4weeks" ) { $newtime = 4 * 7 * 24 * 60 * 60; }
	elsif ( $time eq "5weeks" ) { $newtime = 5 * 7 * 24 * 60 * 60; }
	elsif ( $time eq "6weeks" ) { $newtime = 6 * 7 * 24 * 60 * 60; }
	elsif ( $time eq "7weeks" ) { $newtime = 7 * 7 * 24 * 60 * 60; }
	elsif ( $time eq "8weeks" ) { $newtime = 8 * 7 * 24 * 60 * 60; }
	elsif ( $time eq "1month" ) { $newtime = 1 * 31 * 24 * 60 * 60; }
	elsif ( $time eq "2months" ) { $newtime = 2 * 31 * 24 * 60 * 60; }
	elsif ( $time eq "4months" ) { $newtime = 4 * 31 * 24 * 60 * 60; }
	elsif ( $time eq "6months" ) { $newtime = 6 * 31 * 24 * 60 * 60; }
	elsif ( $time eq "1year" ) { $newtime = 365 * 24 * 60 * 60; }

	return $newtime;
}

1;
